using System;
using System.IO;
using System.Windows.Forms;
using System.Text.RegularExpressions;
using System.Text;
using System.Collections;

namespace RegExRep
{
	/// <summary>
	///         
	/// (  ,  ).
	/// </summary>
	public class AscRegExpParser
	{
		protected Regex m_rx;
		protected string m_sReplaceWith;

		/// <summary>
		///   .
		/// </summary>
		/// <param name="sPatern">  .</param>
		/// <param name="sReplaceWith">  .</param>
		/// <param name="Options">.</param>
		public AscRegExpParser(string sPatern, string sReplaceWith, RegexOptions Options)
		{
			m_rx = new Regex(sPatern, Options);
			m_sReplaceWith = sReplaceWith;
		}

		/// <summary>
		///     .
		/// </summary>
		/// <param name="sFilePath"> .</param>
		/// <returns> .</returns>
		public string ReadFile(string sFilePath)
		{
			using(StreamReader sr = new StreamReader(sFilePath, Encoding.Default, false, 3000000))
				return sr.ReadToEnd();
		}

		/// <summary>
		///       , 
		///    . (  )
		/// </summary>
		/// <param name="sText">     .</param>
		/// <returns>    .</returns>
		/// <remarks>     
		/// .</remarks>
		public MatchCollection Parse(string sText)
		{
			return m_rx.Matches(sText);
		}

		/// <summary>
		///        .
		/// </summary>
		/// <param name="sFilePath"> .</param>
		/// <returns>    .</returns>
		/// <remarks>     
		/// .</remarks>
		public MatchCollection ReadAndParsFile(string sFilePath)
		{
			return Parse(ReadFile(sFilePath));
		}

		/// <summary>
		///       .
		/// </summary>
		/// <param name="sFilePath">   .</param>
		/// <remarks>     
		/// .</remarks>
		public void Replace(string sFilePath, int iReplaceCount)
		{
			string sText = null;
			if(iReplaceCount > 0)
				sText = m_rx.Replace(ReadFile(sFilePath), m_sReplaceWith, iReplaceCount);
			else
				sText = m_rx.Replace(ReadFile(sFilePath), m_sReplaceWith);
			using(StreamWriter sw = new StreamWriter(sFilePath, false, Encoding.Default))
				sw.Write(sText);
		}
	}

	/// <summary>
	///     .
	/// </summary>
	public class AscSearchEngine : IEnumerable
	{
		public void ScanDir(string sPath, string sPattern)
		{
			GC.Collect();
			m_Root = new AscDir(null, sPath, sPattern);
			GC.Collect();
		}

		public void FillTreeView(TreeView tv)
		{
			GC.Collect();
			tv.Nodes.Clear();
			FillNode(tv.Nodes, m_Root);
			GC.Collect();
		}

		/// <summary>
		/// ,     .
		/// </summary>
		public const int ciDir = 0, ciDirSel = 1, ciFile = 2, ciFileSel = 2;

		/// <summary>
		///      TreeView.
		/// </summary>
		private void FillNode(TreeNodeCollection tnc, AscDir dir)
		{
			TreeNode tn;
			foreach(AscDir ad in dir.m_SubDir)
			{
				//  ,      .
				if(ad.m_iFiles > 0)
				{
					tn = new TreeNode(ad.m_sName);
					tn.SelectedImageIndex = ciDirSel;
					tn.ImageIndex = ciDir;
					tnc.Add(tn);
					FillNode(tn.Nodes, ad);
				}
			}
			if(dir.m_Files != null)
				foreach(string s in dir.m_Files)
				{
					tn = new TreeNode(s);
					tn.SelectedImageIndex = ciFileSel;
					tn.ImageIndex = ciFile;
					tnc.Add(tn);
				}
		}

		public int DirsCount{ get{ return m_Root.m_iDirs; } }
		public int FilesCount{ get{ return m_Root.m_iFiles; } }

		public delegate void FileCallback(string sFileName);
		
		public void IterateNodes(FileCallback Callback)
		{
			m_Root.InternalIterateNode(Callback, "");
		}

		//   IEnumerable
		//      GetEnumerator
		IEnumerator IEnumerable.GetEnumerator()
		{	//       IEnumerator
			return (IEnumerator)new AscDir.Iterator(m_Root);
		}

		/// <summary>
		///    . 
		/// </summary>
		private class AscDir
		{
			/// <summary>
			///      
			///  AscDir       .
			/// </summary>
			public class Iterator : IEnumerator 
			{
				AscDir m_ad; //  .
				AscDir m_adRoot;
				int m_iCurrDir;
				int m_iCurrFile;
				Stack aryDirStak = new Stack(20);
				public Iterator(AscDir ad)
				{
					m_adRoot = ad;
					Reset();
				}

				// IEnumerator
				public void Reset()
				{
					m_iCurrDir = 0;
					m_iCurrFile = -1;
					m_ad = m_adRoot;
				}

				public bool MoveNext()
				{
					if(m_ad.m_Files == null || m_iCurrFile + 1 >= m_ad.m_Files.Length)
					{
						if(m_iCurrDir < m_ad.m_SubDir.Length)
						{
							aryDirStak.Push(m_iCurrDir);
							m_ad = m_ad.m_SubDir[m_iCurrDir];
							m_iCurrDir = 0;
							m_iCurrFile = -1;
							return MoveNext();
						}
						else
						{
							if(aryDirStak.Count > 0)
							{
								m_iCurrDir = (int)aryDirStak.Pop() + 1;
								m_ad = m_ad.m_ParentDir;
								m_iCurrFile = m_ad.m_Files == null 
									? 0 : m_ad.m_Files.Length;
								return MoveNext();
							}
							return false;
						}
					}
					else
					{
						m_iCurrFile++;
						if(m_iCurrFile >= m_ad.m_Files.Length)
							return MoveNext();
						return true;
					}
				}

				public object Current 
				{
					get
					{
						return m_ad.m_Files[m_iCurrFile];
					}
				}
			}

			/// <summary>
			///   .   
			///   . 
			/// </summary>
			internal AscDir(AscDir ParentDir, string sPath, string sPattern)
			{
				m_ParentDir = ParentDir; //   .
				//  DirectoryInfo    
				//   .       .
				DirectoryInfo di = new DirectoryInfo(sPath);
				//   .       
				//   DirectoryInfo,       
				//  .
				m_sName = di.Name;
				//   ()  (  ).
				DirectoryInfo[] adi = di.GetDirectories();
				//       AscDir.
				m_SubDir = new AscDir[adi.Length];
				//           AscDir.
				//     ,  ,   , 
				//  .
				for(int i = 0; i < m_SubDir.Length; i++)
					//    AscDir  ,    
					//   ,    .
					m_SubDir[i] = new AscDir(this, sPath + @"\" + adi[i].Name, sPattern);

				//        
				//    .

				//     .     
				//     ;. ,    
				//  *.txt;*.htm*;*.asp.
				//    Split  string.
				string[] asPatterns = sPattern.Split(';');
				//        .
				//    ...
				foreach(string s in asPatterns)
					//       .
					//    GetFiles  DirectoryInfo.
					//  ,    ,  
					//    .
					AppendFileNames(di.GetFiles(s));

				//       (  
				//   ).

				//       
				//   .   ,      
				//   m_Files      
				// AppendFileNames.    .      
				//   ,        m_iFiles
				//   NULL.      
				//  .
				m_iFiles = m_Files == null ? 0 : m_Files.Length;
				//     .
				m_iDirs = m_SubDir.Length;

				//       .
				//      AscDir,    
				//   .     
				// ,      m_iFiles   
				//     ,     
				//  m_iFiles     .
				foreach(AscDir SubDir in m_SubDir)
				{
					m_iFiles += SubDir.m_iFiles;
					m_iDirs += SubDir.m_iDirs;
				}
			}

			/// <summary>
			///       FileInfo  
			///   (m_Files).          
			/// ,    private.
			/// </summary>
			private void AppendFileNames(FileInfo[] afi)
			{
				if(afi != null && afi.Length > 0) //    .
				{
					//         m_Files.
					int iCurLen = (m_Files == null ? 0 : m_Files.Length);
					//      ,  
					//   .       
					//  ArrayList,   ,   ,
					//   .
					string[] sTmp = new string[iCurLen + afi.Length];
					//     ...
					if(m_Files != null)
						//       .
						Array.Copy(m_Files, sTmp, m_Files.Length);
					//  iCurLen     .
					//        , 
					//   ,       .   
					//        .
					//    .
					int i = iCurLen;
					//           .
					foreach(FileInfo fi in afi)
					{
						sTmp[i] = fi.Name;
						i++;
					}
					//   m_Files    .
					m_Files = sTmp;
				}
			}

			internal void InternalIterateNode(FileCallback Callback, string sPath)
			{
				//    ...
				if(m_ParentDir != null)
					sPath += m_sName + '\\'; //    .
				//   ,     ().
				foreach(AscDir Dir in m_SubDir)
					Dir.InternalIterateNode(Callback, (string)sPath.Clone());
				//      ...
				if(m_Files != null)
					foreach(string File in m_Files) //  ..
						Callback(sPath + File); //     callback-.
			}

			//    (AscDir)   .
			//          NULL.
			protected AscDir m_ParentDir;
			//     AscDir ( ).
			internal string m_sName;
			//    (   AscDir).
			internal AscDir[] m_SubDir;
			//        
			//   .      .
			internal string[] m_Files;
			//  . ,     .
			//  .
			internal int m_iDirs = 0;
			//       .
			//  .
			internal int m_iFiles = 0;
		}

		private AscDir m_Root;
	}
}
