#include "stdafx.h"

#include <shlobj.h>
#include <vcclr.h>
#include <comdef.h>

#include "Aist.Utils.h"

using namespace System::Runtime::InteropServices;

namespace Aist
{
  namespace Utils
  {
    //     SHBrowseForFolder
    //
    static int CALLBACK BrowseCallbackProc(HWND hWnd, UINT uMsg, LPARAM lParam, LPARAM lpData)
    {
      GCHandle handle = GCHandle::op_Explicit(IntPtr(lpData));
      BrowseFolderDialog* pDlg = __try_cast<BrowseFolderDialog*>(handle.Target);
      return pDlg->BrowseCallback(hWnd, uMsg, lParam);
    }

    //  
    //
    int BrowseFolderDialog::BrowseCallback(HWND hWnd, UINT uMsg, LPARAM lParam)
    {
      switch (uMsg) {
        case BFFM_INITIALIZED:
          OnInitialize(hWnd);
          break;
        case BFFM_SELCHANGED:
          OnSelChanged(hWnd, lParam);
          break;
        case BFFM_VALIDATEFAILED:
          if (ValidateFailed != NULL) {
#ifdef _UNICODE
            String* str = Marshal::PtrToStringUni(lParam);
#else
            String* str = Marshal::PtrToStringAnsi(lParam);
#endif
            ValidateFailedEventArgs* args = new ValidateFailedEventArgs(str);
            ValidateFailed(this, args);
            if (args->Cancel)
              return 1;
          }
      }

      return 0;
    }

    //  
    //
    void BrowseFolderDialog::OnInitialize(HWND hWnd)
    {
      const wchar_t __pin* strInitialFolder = PtrToStringChars(m_strInitialFolderName);
      _bstr_t bstr(strInitialFolder);

      SendMessage(hWnd, BFFM_SETSELECTION, TRUE, LPARAM(LPCTSTR(bstr)));
    }

    //  
    //
    void BrowseFolderDialog::OnSelChanged(HWND hWnd, LPARAM lParam)
    {
      TCHAR szPath[_MAX_PATH];
      ::SHGetPathFromIDList(LPITEMIDLIST(lParam), szPath);
      ::SendMessage(hWnd, BFFM_SETSTATUSTEXT, NULL, LPARAM(szPath));
    }

    //   
    //
    DialogResult BrowseFolderDialog::ShowDialog(IWin32Window* parent)
    {
      DialogResult result = DialogResult::Cancel;

      m_strFolderName = String::Empty;

      LPMALLOC pMalloc;
      if (::SHGetMalloc(&pMalloc) == S_OK) {
        TCHAR szDisplayName[_MAX_PATH];
        const wchar_t __pin* strTitle = PtrToStringChars(m_strDescription);
        _bstr_t bstr(strTitle);
        BROWSEINFO info;
        ZeroMemory(&info, sizeof(info));
        info.hwndOwner = HWND(parent->Handle.ToInt32());
        info.lpszTitle = bstr;
        info.pszDisplayName = szDisplayName;
        info.ulFlags = m_Flags | BIF_STATUSTEXT;
        info.lpfn = BrowseCallbackProc;
        info.lParam = IntPtr(GCHandle::Alloc(this)).ToInt32();

        if (m_strRoot->Length > 0) {
          IShellFolder *pDF;
          if (::SHGetDesktopFolder(&pDF) == S_OK) {
            LPITEMIDLIST pIdl = NULL;
            ULONG chEaten;
            ULONG dwAttributes;

            const wchar_t __pin* strRoot = PtrToStringChars(m_strRoot);
            LPOLESTR pOleStr = _bstr_t(strRoot);
            pDF->ParseDisplayName(NULL, NULL, pOleStr, &chEaten, &pIdl, &dwAttributes);
            pDF->Release();

            info.pidlRoot = pIdl;
          }
        }

        LPITEMIDLIST pidl = ::SHBrowseForFolder(&info);

        if (pidl != NULL) {
          TCHAR szPath[_MAX_PATH];
          if (::SHGetPathFromIDList(pidl, szPath)) {
            m_strFolderName = szPath;
            result = DialogResult::OK;
          }
          pMalloc->Free(pidl);
        }
        if (info.pidlRoot != NULL)
          pMalloc->Free(LPVOID(info.pidlRoot));
        pMalloc->Release();
      }

      return result;
    }
  }
}
